@@ -77,19 +77,22 @@ Takes the pull-request as only argument and must return a directory."
                     ((or (magit-gitdir) (forge-repository-at-point))
                      "Forge does not yet track this repository")
                     ("Not inside a Git repository")))
-    ("/a" forge-add-repository
+    ("/ a" forge-add-repository
      :description (lambda () (let ((repo (forge-get-repository :stub?)))
                           (if (or (not repo)
                                   (eq (oref repo condition) :tracked))
                             "track some repo"
                           "track this repository"))))
-    ("/M" "merge with api" forge-merge
+    ("c f" "fork this repository" forge-fork
+     :if-not (##forge-get-repository :tracked?))
+    ("/ M" "merge with api" forge-merge
      :if (##forge-get-repository :tracked?)
      :level 7)]]
   [forge--lists-group
    ["Visit"
     :inapt-if-not (##forge-get-repository :tracked?)
     ("v t" "topic"          forge-visit-topic)
+    ("v u" "topic from url" forge-visit-topic-from-url :level 0)
     ("v d" "discussion"     forge-visit-discussion)
     ("v i" "issue"          forge-visit-issue)
     ("v p" "pull-request"   forge-visit-pullreq)]
@@ -144,7 +147,6 @@ repository cannot be determined, instead invoke `forge-add-repository'."
                  (if (forge-get-repository :tracked?)
                      "forge topics"
                    "new forge repository"))
-  (declare (interactive-only nil))
   (interactive)
   (if-let ((repo (forge-get-repository :tracked?)))
       (forge--pull repo)
@@ -173,17 +175,18 @@ repository cannot be determined, instead invoke `forge-add-repository'."
   (magit-git-fetch (oref repo remote) (magit-fetch-arguments)))
 
 (defun forge--maybe-git-fetch (repo &optional buffer)
-  (if (buffer-live-p buffer)
-      (with-current-buffer buffer
-        (if (and (derived-mode-p 'magit-mode)
-                 (forge-repository-equal (forge-get-repository :stub?) repo)
-                 (magit-toplevel))
-            (magit-git-fetch (oref repo remote) (magit-fetch-arguments))
-          (magit-refresh-buffer)))
-    (when-let ((worktree (forge-get-worktree repo)))
-      (let ((default-directory worktree)
-            (magit-inhibit-refresh t))
-        (magit-git-fetch (oref repo remote) (magit-fetch-arguments))))))
+  (cond-let
+    ((buffer-live-p buffer)
+     (with-current-buffer buffer
+       (if (and (derived-mode-p 'magit-mode)
+                (forge-repository-equal (forge-get-repository :stub?) repo)
+                (magit-toplevel))
+           (magit-git-fetch (oref repo remote) (magit-fetch-arguments))
+         (magit-refresh-buffer))))
+    ([worktree (forge-get-worktree repo)]
+     (let ((default-directory worktree)
+           (magit-inhibit-refresh t))
+       (magit-git-fetch (oref repo remote) (magit-fetch-arguments))))))
 
 ;;;###autoload(autoload 'forge-pull-notifications "forge-commands" nil t)
 (transient-define-suffix forge-pull-notifications ()
@@ -205,8 +208,7 @@ repository cannot be determined, instead invoke `forge-add-repository'."
                            (forge--get-github-repository)))
   (interactive
    (list (read-number "Pull topic: "
-                      (and-let* ((topic (forge-current-topic)))
-                        (oref topic number)))))
+                      (and$ (forge-current-topic) (oref $ number)))))
   (forge--pull-topic (forge-get-repository :tracked) number))
 
 ;;;###autoload(autoload 'forge-pull-this-topic "forge-commands" nil t)
@@ -365,16 +367,11 @@ commit, and for a file."
     (user-error "Nothing to browse here")))
 
 (defun forge--browse-target ()
-  (or (and-let* ((branch (magit--painted-branch-at-point)))
-        (forge-get-url :branch branch))
-      (and-let* ((commit (magit-commit-at-point)))
-        (forge-get-url :commit commit))
-      (and-let* ((branch (magit-branch-at-point)))
-        (forge-get-url :branch branch))
-      (and-let* ((remote (magit-remote-at-point)))
-        (forge-get-url :remote remote))
-      (and-let* ((file (magit-file-at-point)))
-        (forge-get-url :blob nil file))
+  (or (and$ (magit--painted-branch-at-point) (forge-get-url :branch $))
+      (and$ (magit-commit-at-point)          (forge-get-url :commit $))
+      (and$ (magit-branch-at-point)          (forge-get-url :branch $))
+      (and$ (magit-remote-at-point)          (forge-get-url :remote $))
+      (and$ (magit-file-at-point)            (forge-get-url :blob nil $))
       (forge-post-at-point)
       (forge-current-topic)
       (and (or magit-buffer-file-name buffer-file-name)
@@ -418,13 +415,14 @@ commit, and for a file."
 
 (cl-defmethod forge-get-url ((_(eql :commit)) commit)
   (let ((repo (forge-get-repository :stub)))
-    (unless (magit-list-containing-branches
-             commit "-r" (concat (oref repo remote) "/*"))
-      (if-let* ((branch (car (magit-list-containing-branches commit "-r")))
-                (remote (car (magit-split-branch-name branch))))
-          (setq repo (forge-get-repository :stub remote))
-        (message "%s does not appear to be available on any remote.  %s"
-                 commit "You might have to push it first.")))
+    (cond-let*
+      ((magit-list-containing-branches
+        commit "-r" (concat (oref repo remote) "/*")))
+      ([branch (car (magit-list-containing-branches commit "-r"))]
+       [remote (car (magit-split-branch-name branch))]
+       (setq repo (forge-get-repository :stub remote)))
+      ((message "%s does not appear to be available on any remote.  %s"
+                commit "You might have to push it first.")))
     (forge--format repo 'commit-url-format
                    `((?r . ,(magit-commit-p commit))))))
 
@@ -537,6 +535,20 @@ lifts the limitation to active pull-requests."
   (interactive (list (forge-read-pullreq "View pull-request")))
   (forge-topic-setup-buffer (forge-get-pullreq pull-request)))
 
+;;;###autoload
+(defun forge-visit-topic-from-url (url)
+  "Visit the topic specified by web URL."
+  (interactive (list (read-string "Topic URL: ")))
+  (if (string-match
+       "/\\(issues\\|pull\\|discussions\\|merge_requests\\)/\\([0-9]+\\)\\'"
+       url)
+      (forge-topic-setup-buffer
+       (forge-get-topic (forge-get-repository
+                         (substring url 0 (match-beginning 1))
+                         nil :tracked)
+                        (string-to-number (match-string 2 url))))
+    (user-error "Not recognized as a topic URL: %s" url)))
+
 ;;;###autoload
 (defun forge-visit-this-topic (&optional menu)
   "Visit the topic at point.
@@ -560,10 +572,11 @@ With prefix argument MENU, also show the topic menu."
     (cond
      ((and (eq transient-current-command 'forge-repositories-menu)
            (forge-get-repository repo nil :tracked?))
-      (if-let ((buffer (get-buffer (forge-topics-buffer-name repo))))
-          (progn (switch-to-buffer buffer)
-                 (transient-setup 'forge-topics-menu))
-        (forge-list-topics repo)))
+      (cond-let
+        ([buffer (get-buffer (forge-topics-buffer-name repo))]
+         (switch-to-buffer buffer)
+         (transient-setup 'forge-topics-menu))
+        ((forge-list-topics repo))))
      (worktree
       (magit-status-setup-buffer worktree))
      ((forge-get-repository repo nil :tracked?)
@@ -576,9 +589,9 @@ With prefix argument MENU, also show the topic menu."
   "Create a new discussion for the current repository."
   (interactive
    (list (forge-read-topic-category nil "Category for new discussion")))
-  (forge--setup-post-buffer nil #'forge--submit-create-discussion
+  (forge--setup-post-buffer 'new-discussion #'forge--submit-create-discussion
     "new-discussion" "Create new discussion on %p"
-   `((forge--buffer-category ,category))))
+    `((forge--buffer-category ,category))))
 
 (defun forge-create-issue (template)
   "Create a new issue for the current repository."
@@ -588,14 +601,14 @@ With prefix argument MENU, also show the topic menu."
       ('redirect (browse-url .url))
       ('forge-discussion (forge-create-discussion .category))
       ('forge-issue
-       (forge--setup-post-buffer nil #'forge--submit-create-issue
+       (forge--setup-post-buffer 'new-issue #'forge--submit-create-issue
          "new-issue" "Create new issue on %p"
          `((forge--buffer-template ,template)))))))
 
 (defun forge-create-pullreq (source target)
   "Create a new pull-request for the current repository."
   (interactive (forge-create-pullreq--read-args))
-  (forge--setup-post-buffer nil #'forge--submit-create-pullreq
+  (forge--setup-post-buffer 'new-pullreq #'forge--submit-create-pullreq
     "new-pullreq" "Create new pull-request on %p"
     `((forge--buffer-base-branch ,target)
       (forge--buffer-head-branch ,source)
@@ -623,11 +636,11 @@ With prefix argument MENU, also show the topic menu."
                    "Source branch"
                    (magit-list-remote-branch-names)
                    nil t nil 'magit-revision-history
-                   (or (and-let* ((d (magit-branch-at-point)))
+                   (or (and-let ((d (magit-branch-at-point)))
                          (if (magit-remote-branch-p d)
                              d
                            (magit-get-push-branch d t)))
-                       (and-let* ((d (magit-get-current-branch)))
+                       (and-let ((d (magit-get-current-branch)))
                          (if (magit-remote-branch-p d)
                              d
                            (magit-get-push-branch d t))))))
@@ -658,10 +671,10 @@ point is currently on."
   (let* ((quote (cond
                  ((not (magit-section-match 'post)) nil)
                  ((use-region-p)
-                  (magit--buffer-string (region-beginning) (region-end)))
+                  (buffer-str (region-beginning) (region-end)))
                  (quote
                   (with-slots (content end) (magit-current-section)
-                    (magit--buffer-string content end t)))))
+                    (string-trim (buffer-str content end))))))
          (quote (and quote
                      (lambda ()
                        (goto-char (point-max))
@@ -770,8 +783,8 @@ Please see the manual for more information."
   (interactive (list (forge-read-pullreq "Branch pull request")))
   (let ((pullreq (forge-get-pullreq pullreq)))
     (if-let ((branch (forge--pullreq-branch-active pullreq)))
-        (progn (message "Branch %S already exists and is configured" branch)
-               branch)
+        (prog1 branch
+          (message "Branch %S already exists and is configured" branch))
       (forge--branch-pullreq pullreq)
       (forge-refresh-buffer))))
 
@@ -795,7 +808,7 @@ Please see the manual for more information."
          (branch-n (format "pr-%s" number))
          (branch (or (forge--pullreq-branch-internal pullreq) branch-n))
          (pullreq-ref (format "refs/pullreqs/%s" number)))
-    (cond ((and-let* ((pr-branch (oref pullreq head-ref)))
+    (cond ((and-let ((pr-branch (oref pullreq head-ref)))
              (string-search ":" pr-branch))
            ;; Such a branch name would be invalid.  If we encounter
            ;; it anyway, then that means that the source branch and
@@ -803,7 +816,7 @@ Please see the manual for more information."
            ;; longer does this, but we nevertheless have to deal
            ;; with merge-requests that have been lost in time.
            (error "Cannot check out this merge-request because %s"
-                  "on old Gitlab version discarded the source branch"))
+                  "an old Gitlab version discarded the source branch"))
           ((not (eq (oref pullreq state) 'open))
            (magit-git "branch" "--force" branch pullreq-ref))
           (t
@@ -949,6 +962,44 @@ information."
       (user-error "The empty string isn't a valid path"))
     path))
 
+;;;###autoload (autoload 'forge-push-to-unnamed-pullreq "forge-commands" nil t)
+(transient-define-suffix forge-push-to-unnamed-pullreq (args)
+  "Push the current branch to the branch on the contributor's fork.
+
+Usually a maintainer would use `magit-push-current-to-pushremote' to
+push to the branch, the contributor asks to be merged.  That does not
+work if they did not create a dedicated branch and instead committed
+directly to \"main\", or some other branch, that also exists in the
+upstream repository.
+
+If this is the case then the branch, which is used to check out the
+pull-request locally, is named \"pr-N\" (where N is the pull-request
+number) and this command is made available as a substitute in the
+`magit-push' menu."
+  :if (lambda ()
+        (and-let ((branch (magit-get-current-branch)))
+          (and (forge-get-pullreq :branch branch)
+               (string-match-p "\\`pr-[0-9]+\\'" branch))))
+  :description (lambda ()
+                 (and-let* ((branch (magit-get-current-branch))
+                            (pullreq (forge-get-pullreq :branch branch)))
+                   (format "contributor's %s branch"
+                           (magit--propertize-face
+                            (format "%s/%s"
+                                    (oref pullreq head-user)
+                                    (oref pullreq head-ref))
+                            'magit-branch-remote))))
+  (interactive (list (magit-push-arguments)))
+  (cond-let*
+    ([branch (magit-get-current-branch)]
+     [pullreq (forge-get-pullreq :branch branch)]
+     (run-hooks 'magit-credential-hook)
+     (magit-run-git-async "push" "-v"
+                          (delete "--tags" (delete "--follow-tags" args))
+                          (oref pullreq head-user)
+                          (format "%s:%s" branch (oref pullreq head-ref))))
+    ((error "Checked out branch is not an unnamed pull-request branch"))))
+
 ;;; Marks
 
 (defun forge-create-mark (name face description)
@@ -1266,15 +1317,15 @@ upstream remote."
                      :scope (forge-add-repository--scope repo)))
    (t
     (when-let*
-        (((not (eq limit :selective)))
-         ((magit-git-config-p "forge.autoPull" t))
+        ((_(not (eq limit :selective)))
+         (_(magit-git-config-p "forge.autoPull" t))
          (remote  (oref repo remote))
          (refspec (oref repo pullreq-refspec))
          (default-directory (forge-get-worktree repo))
-         ((and (not (member refspec (magit-get-all "remote" remote "fetch")))
-               (or (eq forge-add-pullreq-refspec t)
-                   (and (eq forge-add-pullreq-refspec 'ask)
-                        (y-or-n-p (format "Also add %S refspec? " refspec)))))))
+         (_(and (not (member refspec (magit-get-all "remote" remote "fetch")))
+                (or (eq forge-add-pullreq-refspec t)
+                    (and (eq forge-add-pullreq-refspec 'ask)
+                         (y-or-n-p (format "Also add %S refspec? " refspec)))))))
       (magit-call-git "config" "--add"
                       (format "remote.%s.fetch" remote)
                       refspec))
@@ -1380,9 +1431,9 @@ when that happens, because given how the APIs work, this would be too
 expensive."
   (interactive
    (list (if-let* ((topics (magit-region-values '(issue pullreq) t))
-                   ((magit-confirm 'remove-topics-locally nil
-                      "Delete %d topics locally" nil
-                      (mapcar #'forge--format-topic-line topics))))
+                   (_(magit-confirm 'remove-topics-locally nil
+                       "Delete %d topics locally" nil
+                       (mapcar #'forge--format-topic-line topics))))
              topics
            (forge-read-topic "Delete topic LOCALLY only"))))
   (if (listp topic)
@@ -1445,8 +1496,13 @@ context."
 ;;; _
 ;; Local Variables:
 ;; read-symbol-shorthands: (
-;;   ("partial" . "llama--left-apply-partially")
-;;   ("rpartial" . "llama--right-apply-partially"))
+;;   ("and$"          . "cond-let--and$")
+;;   ("and-let"       . "cond-let--and-let")
+;;   ("if-let"        . "cond-let--if-let")
+;;   ("when-let"      . "cond-let--when-let")
+;;   ("buffer-string" . "buffer-string")
+;;   ("buffer-str"    . "forge--buffer-substring-no-properties")
+;;   ("partial"       . "llama--left-apply-partially"))
 ;; End:
 (provide 'forge-commands)
 ;;; forge-commands.el ends here
